from django.conf import settings
from simple_translation.translation_pool import translation_pool

def get_language_from_request(request):
    return request.REQUEST.get('language', getattr(request, 'LANGUAGE_CODE', settings.LANGUAGE_CODE))

def get_preferred_translation_from_request(obj, request):
    language = getattr(request, 'LANGUAGE_CODE', settings.LANGUAGE_CODE)
    if not hasattr(obj, 'translations'):
        translation_pool.annotate_with_translations(obj)
    for translation in obj.translations:
        if translation.language == language:
            return translation
    return obj.translations[0]
    
def get_preferred_translation_from_lang(obj, language):
    if not hasattr(obj, 'translations'):
        translation_pool.annotate_with_translations(obj)
    for translation in obj.translations:
        if translation.language == language:
            return translation
    return obj.translations[0]
    
def get_translation_filter(model, **kwargs):
    info = translation_pool.get_info(model)
    join_filter = info.translation_join_filter
    filter_dict = {}
    for key, value in kwargs.items():
        filter_dict['%s__%s' % (join_filter, key)] = value
    return filter_dict
    
def get_translation_filter_language(model, language, **kwargs):
    info = translation_pool.get_info(model)
    kwargs[info.language_field] = language
    return get_translation_filter(model, **kwargs)        

def get_translation_manager(obj):
    info = translation_pool.get_info(obj.__class__)
    return getattr(obj, info.translations_of_accessor)

def get_translation_queryset(obj):
    return get_translation_manager(obj).all()  

def get_translated_model(model):
    return translation_pool.get_info(model).translated_model   


from django.conf import settings
from django.conf.urls.defaults import *
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
    (r'^jsi18n/(?P<packages>\S+?)/$', 'django.views.i18n.javascript_catalog'),
)


urlpatterns += patterns('', 
    url(r'^', include('simple_translation.test.testapp.urls'))
)


import datetime
from django.core.urlresolvers import reverse
from django.conf import settings
from django.contrib.auth.models import User
from django.template import Template, Context
from django.test.client import RequestFactory
from simple_translation.test.testcases import SimpleTranslationBaseTestCase

class SimpleTranslationTestCase(SimpleTranslationBaseTestCase):
    
    def setUp(self):
        self.request_factory =  RequestFactory()
           
    def test_01_test_translated_urls(self):
        
        old_urlconf  = settings.ROOT_URLCONF
        settings.ROOT_URLCONF = 'simple_translation.test.testapp.translated_urls'
        old_middleware = settings.MIDDLEWARE_CLASSES
        settings.MIDDLEWARE_CLASSES = old_middleware +[
            'simple_translation.middleware.MultilingualGenericsMiddleware']

        published_at = datetime.datetime.now() - datetime.timedelta(hours=-1)
        en_title, entry = self.create_entry_with_title(title='english', published_at=published_at)
        
        de_title = self.create_entry_title(entry, title='german', language='de', published_at=published_at)
            
        index = reverse('entry_archive_index')
        
        en_index = reverse('en:entry_archive_index')
        de_index = reverse('de:entry_archive_index')
        
        self.assertEquals(index, '/')        
        self.assertEquals(en_index, '/en/')        
        self.assertEquals(de_index, '/de/')
        
        response = self.client.get(index)
        self.assertContains(response, 'english')

        response = self.client.get(en_index)
        self.assertContains(response, 'english')
        self.assertNotContains(response, 'german')

        response = self.client.get(de_index)
        self.assertContains(response, 'german')
        self.assertNotContains(response, 'english')
        
        settings.ROOT_URLCONF = old_urlconf
        settings.MIDDLEWARE_CLASSES = old_middleware
        
    def test_02_test_no_translated_urls_with_middleware(self):
        
        old_middleware = settings.MIDDLEWARE_CLASSES
        settings.MIDDLEWARE_CLASSES = old_middleware +[
            'simple_translation.middleware.MultilingualGenericsMiddleware']
            
        published_at = datetime.datetime.now() - datetime.timedelta(hours=-1)
        en_title, entry = self.create_entry_with_title(title='english', published_at=published_at)
        
        de_title = self.create_entry_title(entry, title='german', language='de', published_at=published_at)
            	
        index = reverse('entry_archive_index')
        
        response = self.client.get(index)
        self.assertContains(response, 'english')
        
        settings.MIDDLEWARE_CLASSES = old_middleware
        
    def test_03_test_no_translated_urls_without_middleware(self):
            
        published_at = datetime.datetime.now() - datetime.timedelta(hours=-1)
        en_title, entry = self.create_entry_with_title(title='english', published_at=published_at)
        
        de_title = self.create_entry_title(entry, title='german', language='de', published_at=published_at)
                	
        index = reverse('entry_archive_index')
        
        response = self.client.get(index)
        self.assertContains(response, 'english')
        self.assertContains(response, 'german')

    def test_04_test_no_translated_urls_with_locale_middleware(self):
        
        old_middleware = settings.MIDDLEWARE_CLASSES
        settings.MIDDLEWARE_CLASSES = old_middleware + [
            'django.middleware.locale.LocaleMiddleware',
            'simple_translation.middleware.MultilingualGenericsMiddleware']
            
        published_at = datetime.datetime.now() - datetime.timedelta(hours=-1)
        en_title, entry = self.create_entry_with_title(title='english', published_at=published_at)
        
        de_title = self.create_entry_title(entry, title='german', language='de', published_at=published_at)
        
        index = reverse('entry_archive_index')
        
        response = self.client.get(index)
        self.assertContains(response, 'english')
        self.assertNotContains(response, 'german')
        
        settings.MIDDLEWARE_CLASSES = old_middleware
        
    def test_05_test_translated_urls_with_locale_middleware(self):
        
        old_urlconf  = settings.ROOT_URLCONF
        settings.ROOT_URLCONF = 'simple_translation.test.testapp.translated_urls'
        old_middleware = settings.MIDDLEWARE_CLASSES
        settings.MIDDLEWARE_CLASSES = old_middleware +[
            'django.middleware.locale.LocaleMiddleware',
            'simple_translation.middleware.MultilingualGenericsMiddleware']
        
        published_at = datetime.datetime.now() - datetime.timedelta(hours=-1)
        en_title, entry = self.create_entry_with_title(title='english', published_at=published_at)
        
        de_title = self.create_entry_title(entry, title='german', language='de', published_at=published_at)
            
        index = reverse('entry_archive_index')
        
        en_index = reverse('en:entry_archive_index')
        de_index = reverse('de:entry_archive_index')
        
        self.assertEquals(index, '/')        
        self.assertEquals(en_index, '/en/')        
        self.assertEquals(de_index, '/de/')
        
        response = self.client.get(index)
        self.assertContains(response, 'english') # localemiddleware wins
        self.assertNotContains(response, 'german')

        response = self.client.get(en_index)
        self.assertContains(response, 'english')
        self.assertNotContains(response, 'german')

        response = self.client.get(de_index)
        self.assertContains(response, 'german')
        self.assertNotContains(response, 'english') # generics middleware wins
        
        settings.ROOT_URLCONF = old_urlconf
        settings.MIDDLEWARE_CLASSES = old_middleware
        
    def test_06_admin_edit_translated_entry(self):
        
        superuser = User(username="super", is_staff=True, is_active=True, 
            is_superuser=True)
        superuser.set_password("super")
        superuser.save()
        
        self.client.login(username='super', password='super')
        
        published_at = datetime.datetime.now() - datetime.timedelta(hours=-1)
        en_title, entry = self.create_entry_with_title(title='english', published_at=published_at)
        
        de_title = self.create_entry_title(entry, title='german', language='de', published_at=published_at)
        
        edit_url = reverse('admin:testapp_entry_change', args=(str(entry.pk)))
        
        # edit english(default)
        response = self.client.get(edit_url)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, 'class="simple-translation-current" name="en"' )
        
        # edit english
        response = self.client.get(edit_url, {'language': 'en'})
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, 'class="simple-translation-current" name="en"' )
        
        # edit german
        response = self.client.get(edit_url, {'language': 'de'})
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, 'class="simple-translation-current" name="de"' )
        
        
    def test_07_test_changelist_description(self):
        superuser = User(username="super", is_staff=True, is_active=True, 
            is_superuser=True)
        superuser.set_password("super")
        superuser.save()
        
        self.client.login(username='super', password='super')
        
        published_at = datetime.datetime.now() - datetime.timedelta(hours=-1)
        en_title, entry = self.create_entry_with_title(title='english', published_at=published_at)
        
        de_title = self.create_entry_title(entry, title='german', language='de', published_at=published_at)
            
        list_url = reverse('admin:testapp_entry_changelist')
        response = self.client.get(list_url)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, '<a href="1/?language=en">EN</a>' )
        self.assertContains(response, '<a href="1/?language=de">DE</a>' )
        
    def test_08_test_filters(self):
        published_at = datetime.datetime.now() - datetime.timedelta(hours=-1)
        en_title, entry = self.create_entry_with_title(title='english', published_at=published_at)
        
        de_title = self.create_entry_title(entry, title='german', language='de', published_at=published_at)
        
        request = self.request_factory.get('')
        request.LANGUAGE_CODE = 'en'
        
        ctxt = Context({'entry': entry, 'request': request})
        
        tpl_req = Template('''{% load simple_translation_tags %}
            {% with entry|get_preferred_translation_from_request:request as title %}
                {{ title }}
            {% endwith %}
        ''')
        
        self.assertEquals(tpl_req.render(ctxt).strip(), 'english')
        
        tpl_lang = Template('''{% load simple_translation_tags %}
            {% with entry|get_preferred_translation_from_lang:'de' as title %}
                {{ title }}
            {% endwith %}
        ''')
        self.assertEquals(tpl_lang.render(ctxt).strip(), 'german')



from django.conf import settings
from django.conf.urls.defaults import *

from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
    (r'^jsi18n/(?P<packages>\S+?)/$', 'django.views.i18n.javascript_catalog'),
)

urlpatterns +=  patterns('', url(r'^',
    include('simple_translation.test.testapp.urls', app_name='testapp')
    )
)

for langcode in dict(settings.LANGUAGES).keys():
    urlpatterns +=  patterns('', url(r'^%s/' % langcode,
        include('simple_translation.test.testapp.urls',
            namespace=langcode, app_name='testapp'),
        kwargs={'language_code': langcode}
    )
)




import datetime

from django.db import models
from django.conf import settings

class Entry(models.Model):
    is_published = models.BooleanField()

class EntryTitle(models.Model):
    entry = models.ForeignKey(Entry)
    language = models.CharField(max_length=2, choices=settings.LANGUAGES)
    title = models.CharField(max_length=255)
    slug = models.SlugField(unique=True)
    pub_date = models.DateTimeField(default=datetime.datetime.now)

    def __unicode__(self):
        return self.title
        
    def _get_absolute_url(self):
        language_namespace = 'simple_translation.middleware.MultilingualGenericsMiddleware' in settings.MIDDLEWARE_CLASSES and '%s:' % self.language or ''
        return ('%sentry_detail' % language_namespace, (), {
            'year': self.pub_date.strftime('%Y'),
            'month': self.pub_date.strftime('%m'),
            'day': self.pub_date.strftime('%d'),
            'slug': self.slug
        })
    get_absolute_url = models.permalink(_get_absolute_url)


from django.template.defaultfilters import slugify
from django.test.testcases import TestCase
from simple_translation.test.testapp.models import Entry, EntryTitle

class SimpleTranslationBaseTestCase(TestCase):
        
    def create_entry_with_title(self, title=None, slug=None, language=None, published_at=None):
    	kwargs = {'is_published': True}
        entry = Entry.objects.create(**kwargs)
        entrytitle = self.create_entry_title(entry, title=title, slug=slug, language=language, published_at=published_at)
        return (entrytitle, entry)
        
    def create_entry_title(self, entry, title=None, slug=None, language=None, published_at=None):
        if not title:
            title = 'Entry title'
        slug = slug or slugify(title)
        language = language or 'en'
        return entry.entrytitle_set.create(entry=entry, title=title, slug=slug, language=language, pub_date=published_at)


import os
from functools import partial
from django.utils.translation import ugettext as _

from django.conf import settings
from django.db import router
from django import forms
from django.contrib import admin

from django.contrib.admin.util import unquote, get_deleted_objects, flatten_fieldsets

from django.utils.encoding import force_unicode
from django.utils.functional import curry
from django.http import HttpResponseRedirect, HttpResponse, Http404, \
    HttpResponseBadRequest, HttpResponseForbidden, HttpResponseNotAllowed
from django.shortcuts import render_to_response, get_object_or_404
from django.template.context import RequestContext

from simple_translation.widgets import LanguageWidget
from simple_translation.forms import TranslationModelForm, translation_modelform_factory
from simple_translation.utils import get_language_from_request
from simple_translation.translation_pool import translation_pool

import django

def make_translation_admin(admin):
    
    class RealTranslationAdmin(admin):
        
        form = TranslationModelForm
        
        list_display = ('description', 'languages')
        
        def __init__(self, *args, **kwargs):
            super(RealTranslationAdmin, self).__init__(*args, **kwargs)
            info = translation_pool.get_info(self.model)
            self.translated_model = info.translated_model
            self.translation_of_field = info.translation_of_field
            self.language_field = info.language_field
    
        def description(self, obj):
            return getattr(translation_pool.annotate_with_translations(obj), 'translations', []) \
            	and unicode(translation_pool.annotate_with_translations(obj).translations[0]) or u'No translations'
        
        def languages(self, obj):
                lnk = '<a href="%s/?language=%s">%s</a>'
                trans_list = [ (obj.pk, \
                	getattr(t, self.language_field), getattr(t, self.language_field).upper())
                    	for t in getattr(translation_pool.annotate_with_translations(obj), 'translations') or []]
                return ' '.join([lnk % t for t in trans_list])
        languages.short_description = _('languages')
        languages.allow_tags = True

        def get_translation(self, request, obj):
    
            language = get_language_from_request(request)
 
            if obj:
                
                get_kwargs = {
                    self.translation_of_field: obj,
                    self.language_field: language
                }
    
                try:
                    return self.translated_model.objects.get(**get_kwargs)
                except:
                    return self.translated_model(**get_kwargs)
    
            return self.translated_model(**{self.language_field: language})
                
        def get_form(self, request, obj=None, **kwargs):
            """
            Returns a Form class for use in the admin add view. This is used by
            add_view and change_view.
            """
            if self.declared_fieldsets:
                fields = flatten_fieldsets(self.declared_fieldsets)
            else:
                fields = None
            if self.exclude is None:
                exclude = []
            else:
                exclude = list(self.exclude)
            exclude.extend(kwargs.get("exclude", []))
            exclude.extend(self.get_readonly_fields(request, obj))
            # if exclude is an empty list we pass None to be consistant with the
            # default on modelform_factory
            exclude = exclude or None
            defaults = {
                "form": self.form,
                "fields": fields,
                "exclude": exclude,
                "formfield_callback": curry(self.formfield_for_dbfield, request=request),
            }
            defaults.update(kwargs)
            new_form = translation_modelform_factory(self.model, **defaults)
            current_language = get_language_from_request(request)
            translation_obj = self.get_translation(request, obj)
            new_form.base_fields[self.language_field].widget = LanguageWidget(
                translation_of_obj=obj,
                translation_obj=translation_obj
            )
            new_form.base_fields[self.language_field].initial = current_language

            return new_form

        def save_translated_form(self, request, obj, form, change):
            return form.child_form.save(commit=False)            

        def save_translated_model(self, request, obj, translation_obj, form, change):
            setattr(translation_obj, self.translation_of_field, obj) 
            translation_obj.save()
            
        def save_model(self, request, obj, form, change):
            super(RealTranslationAdmin, self).save_model(request, obj, form, change)
            if hasattr(form, 'child_form'):
                translation_obj = self.save_translated_form(request, obj, form, change)
                self.save_translated_model(request, obj, translation_obj, form, change)
            
        def placeholder_plugin_filter(self, request, queryset):
            language = get_language_from_request(request)
            return queryset.filter(language=language)
            
        def response_change(self, request, obj):
            response = super(RealTranslationAdmin, self).response_change(request, obj)
            language = get_language_from_request(request)
            if response.status_code == 302 and response._headers['location'][1] == request.path:
                location = response._headers['location']
                response._headers['location'] = (location[0], "%s?language=%s" % (location[1], language))
            return response
        
        def response_add(self, request, obj, post_url_continue='../%s/'):
            response = super(RealTranslationAdmin, self).response_add(request, obj, post_url_continue)
            if request.POST.has_key("_continue"):
                language = get_language_from_request(request)
                location = response._headers['location']
                response._headers['location'] = (location[0], "%s?language=%s" % (location[1], language))
            return response
            
        def delete_translation(self, request, object_id, extra_context=None):
    
            language = get_language_from_request(request)
 
            opts = self.model._meta
            translationopts = self.translated_model._meta
            app_label = translationopts.app_label
    
            try:
                obj = self.queryset(request).get(pk=unquote(object_id))
            except self.model.DoesNotExist:
                # Don't raise Http404 just yet, because we haven't checked
                # permissions yet. We don't want an unauthenticated user to be able
                # to determine whether a given object exists.
                obj = None
    
            if not self.has_delete_permission(request, obj):
                raise PermissionDenied
    
            if obj is None:
                raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})
    
            if not len(translation_pool.annotate_with_translations(obj).translations) > 1:
                raise Http404(_('There only exists one translation for this page'))
    
            translationobj = get_object_or_404(self.translated_model, **{self.translation_of_field + '__id': object_id, 'language': language})

            if django.VERSION[1] > 2: # pragma: no cover
                # WARNING: Django 1.3 is not officially supported yet!
                using = router.db_for_read(self.model)
                kwargs = {
                    'admin_site': self.admin_site,
                    'user': request.user,
                    'using': using
                }
            else:
                kwargs = {
                    'admin_site': self.admin_site,
                    'user': request.user,
                }

            deleted_objects, perms_needed = get_deleted_objects([translationobj], translationopts, **kwargs)        
    
            if request.method == 'POST':
                if perms_needed:
                    raise PermissionDenied
    
                message = _('%(obj_name)s with language %(language)s was deleted') % {
                    'language': [name for code, name in settings.LANGUAGES if code == language][0], 'obj_name': force_unicode(translationopts.verbose_name)}
                self.log_change(request, translationobj, message)
                self.message_user(request, message)
    
                translationobj.delete()
    
                if not self.has_change_permission(request, None):
                    return HttpResponseRedirect("../../../../")
                return HttpResponseRedirect("../../")
    
            context = {
                "title": _("Are you sure?"),
                "object_name": force_unicode(translationopts.verbose_name),
                "object": translationobj,
                "deleted_objects": deleted_objects,
                "perms_lacking": perms_needed,
                "opts": translationopts,
                "root_path": self.admin_site.root_path,
                "app_label": app_label,
            }
            context.update(extra_context or {})
            context_instance = RequestContext(request, current_app=self.admin_site.name)
            return render_to_response(self.delete_confirmation_template or [
                "admin/%s/%s/delete_confirmation.html" % (app_label, translationopts.object_name.lower()),
                "admin/%s/delete_confirmation.html" % app_label,
                "admin/delete_confirmation.html"
            ], context, context_instance=context_instance)
            
        def render_change_form(self, request, context, add=False, change=False,  form_url='', obj=None):
            if not self.get_translation(request, obj).pk:
                return super(RealTranslationAdmin, self).render_change_form(request, context, True, change,  form_url, obj)
            else:
                return super(RealTranslationAdmin, self).render_change_form(request, context, add, change,  form_url, obj)
                
        def get_urls(self):
            """Get the admin urls"""
            from django.conf.urls.defaults import patterns, url
            info = "%s_%s" % (self.model._meta.app_label, self.model._meta.module_name)
            pat = lambda regex, fn: url(regex, self.admin_site.admin_view(fn), name='%s_%s' % (info, fn.__name__))
    
            url_patterns = patterns('',
                pat(r'^([0-9]+)/delete-translation/$', self.delete_translation),
            )
    
            url_patterns = url_patterns + super(RealTranslationAdmin, self).get_urls()
            return url_patterns
    return RealTranslationAdmin
    
TranslationAdmin = make_translation_admin(admin.ModelAdmin)

if 'cms' in settings.INSTALLED_APPS:
    from cms.admin.placeholderadmin import PlaceholderAdmin
    PlaceholderTranslationAdmin = make_translation_admin(PlaceholderAdmin)






from django.forms.models import model_to_dict, fields_for_model
from django.forms.models import  ModelForm, ModelFormMetaclass, modelform_factory, model_to_dict
from django.forms.util import ErrorList, ErrorDict
from django.core.exceptions import NON_FIELD_ERRORS
from simple_translation.translation_pool import translation_pool

class TranslationModelFormMetaclass(ModelFormMetaclass):
    
    def __new__(cls, name, bases, attrs):
        formfield_callback = attrs.get('formfield_callback', None)
        try:
            parents = [b for b in bases if issubclass(b, TranslationModelForm)]
        except NameError:
            # We are defining TranslationModelForm itself.
            parents = None
            
        new_class = super(TranslationModelFormMetaclass, cls).__new__(cls, name, bases,
                attrs)
        if not parents:
            return new_class
            
        opts = new_class._meta
        if opts.model:
            if translation_pool.is_registered(opts.model):
                info = translation_pool.get_info(opts.model)
                translated_model = info.translated_model
                new_class.child_form_class = modelform_factory(translated_model,
                    exclude=[info.translation_of_field], formfield_callback=formfield_callback)
                new_class.declared_fields.update(new_class.child_form_class.declared_fields)
                new_class.base_fields.update(new_class.child_form_class.base_fields)
        return new_class
        
class TranslationModelForm(ModelForm):
    __metaclass__ = TranslationModelFormMetaclass
    
    def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
        initial={}, error_class=ErrorList, label_suffix=':',
        empty_permitted=False, instance=None):
        
        model = self._meta.model
        child_model = self.child_form_class._meta.model
        info = translation_pool.get_info(model)
        
        current_language = self.base_fields[info.language_field].initial

        if instance and instance.pk:
            try:
                child_instance = child_model.objects.get(**{
                    info.translation_of_field: instance.pk,
                    info.language_field: current_language})
            except child_model.DoesNotExist:
                child_instance = child_model(**{
                    info.language_field: current_language})
        else:
            child_instance = child_model(**{info.language_field: current_language})
            
        initial.update(model_to_dict(child_instance))
        self.child_form = self.child_form_class(data=data, files=files, auto_id=auto_id, prefix=prefix,
            initial=initial, error_class=error_class, label_suffix=label_suffix,
            empty_permitted=empty_permitted, instance=child_instance)
            
        
        super(TranslationModelForm, self).__init__(data=data, files=files, auto_id=auto_id, prefix=prefix,
            initial=initial, error_class=error_class, label_suffix=label_suffix,
            empty_permitted=empty_permitted, instance=instance)

    def full_clean(self):
        super(TranslationModelForm, self).full_clean()
        if self._errors:
            self.cleaned_data = {}
        self.child_form.full_clean()
        if self.child_form._errors:
            if not self._errors:
                self._errors = ErrorDict()
            for k, v in self.child_form._errors.items():
                if k in self._errors and k != NON_FIELD_ERRORS:
                    # remove existing errors so there are no duplicates
                    del self._errors[k]
            self._update_errors(self.child_form._errors)
            del self.cleaned_data

def translation_modelform_factory(model, form=TranslationModelForm, fields=None, exclude=None,
    formfield_callback=None):
    # Create the inner Meta class. FIXME: ideally, we should be able to
    # construct a ModelForm without creating and passing in a temporary
    # inner class.
    info = translation_pool.get_info(model)
    translated_model = info.translated_model
    translation_fields = [f[0].name for f in translated_model._meta.get_fields_with_model()]
    # Build up a list of attributes that the Meta object will have.
    attrs = {'model': model}
    if fields is not None:
        attrs['fields'] = [ field for field in fields if not field in translation_fields]
    if exclude is not None:
        attrs['exclude'] = exclude

    # If parent form class already has an inner Meta, the Meta we're
    # creating needs to inherit from the parent's inner meta.
    parent = (object,)
    if hasattr(form, 'Meta'):
        parent = (form.Meta, object)
    Meta = type('Meta', parent, attrs)

    # Give this new form class a reasonable name.
    class_name = model.__name__ + 'Form'

    # Class attributes for the new form class.
    form_class_attrs = {
        'Meta': Meta,
        'formfield_callback': formfield_callback
    }

    return TranslationModelFormMetaclass(class_name, (form,), form_class_attrs)



from django.conf import settings
from django.middleware.locale import LocaleMiddleware
from django.utils import translation

from simple_translation.translation_pool import translation_pool

class MultilingualGenericsMiddleware(LocaleMiddleware):
    
    language_fallback_middlewares = ['django.middleware.locale.LocaleMiddleware']
    
    def has_language_fallback_middlewares(self):
        has_fallback = False
        for middleware in self.language_fallback_middlewares: 
            if middleware in settings.MIDDLEWARE_CLASSES:
                has_fallback = True
        return has_fallback
        
    def process_request(self, request):
 	    pass

    def process_view(self, request, view_func, view_args, view_kwargs):
        language = None
        if 'language_code' in view_kwargs:
            # get language and set tralslation
            language = view_kwargs.pop('language_code')
            translation.activate(language)
            request.LANGUAGE_CODE = translation.get_language()
            
        if not language:
            if not self.has_language_fallback_middlewares():
                super(MultilingualGenericsMiddleware, self).process_request(request)
            language = getattr(request, 'LANGUAGE_CODE')
            
        if 'queryset' in view_kwargs:
            filter_expr = None    
            model = view_kwargs['queryset'].model
            if translation_pool.is_registered(model):
                info = translation_pool.get_info(model)
                filter_expr = '%s__%s' % (info.translation_join_filter, info.language_field)
            if translation_pool.is_registered_translation(model):
                info = translation_pool.get_info(model)
                filter_expr = '%s' % info.language_field
            if filter_expr:
                view_kwargs['queryset'] = view_kwargs['queryset'].filter( \
                    **{filter_expr: language}).distinct()
                    
    def process_response(self, request, response):
        if not 'django.middleware.locale.LocaleMiddleware' in settings.MIDDLEWARE_CLASSES:
            return super(MultilingualGenericsMiddleware, self).process_response(request, response)
        return response


from django.conf import settings
from django import template
from django.template.loader import render_to_string
from django.db import models
from simple_translation.translation_pool import translation_pool
from simple_translation.utils import get_preferred_translation_from_request, get_preferred_translation_from_lang

register = template.Library()

def annotate_with_translations(object_or_list):
    return translation_pool.annotate_with_translations(object_or_list)
register.filter(annotate_with_translations)
 
register.filter(get_preferred_translation_from_request)
register.filter(get_preferred_translation_from_lang)
    
def render_language_choices(obj, request):
    if not hasattr(obj, 'translations'):
        annotate_with_translations(obj)
    language = getattr(request, 'LANGUAGE_CODE', settings.LANGUAGE_CODE)
    translations = [translation for translation in obj.translations if translation.language != language]
    opts = obj.__class__._meta
    app_label = opts.app_label
    return render_to_string([
        'simple_translation/%s/%s/language_choices.html' % (app_label, opts.object_name.lower()),
        'simple_translation/%s/language_choices.html' % app_label,
        'simple_translation/language_choices.html'
    ], {'translations': translations})
register.filter(render_language_choices)
        



from django import forms
from django.contrib import admin
from simple_translation.admin import TranslationAdmin

from simple_translation.test.testapp.models import Entry, EntryTitle
from simple_translation.admin import TranslationModelForm

class EntryForm(TranslationModelForm):
    
    class Meta:
        model = Entry
    
class EntryAdmin(TranslationAdmin):
    
    form = EntryForm
    
    prepopulated_fields = {}
        
    list_display = ('description', 'languages', 'is_published')
    list_editable = ('is_published',)
    
    def __init__(self, *args, **kwargs):
        super(EntryAdmin, self).__init__(*args, **kwargs)
        self.prepopulated_fields.update({'slug': ('title',)})
        
    def get_fieldsets(self, request, obj=None):
        fieldsets = super(EntryAdmin, self).get_fieldsets(request, obj=obj)
        fieldsets[0] = (None, {'fields': (
            'language',
            'pub_date',
            'title',
            'slug'
        )})
        return fieldsets
           
admin.site.register(Entry, EntryAdmin)


from django.conf import settings

from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django import forms

from simple_translation.translation_pool import translation_pool

class LanguageWidget(forms.HiddenInput):
	
    class Media:
    	# use getattr until django-cms reccomends django 1.3
    	css = {
    		'all': ['%ssimple_translation/widget.css' % getattr(settings, 'STATIC_URL', getattr(settings, 'MEDIA_URL', ''))]
    	}
    	
    def __init__(self, *args, **kwargs):
        self.translation_of_obj = kwargs.pop('translation_of_obj')
        self.translation_obj = kwargs.pop('translation_obj')
        
        super(LanguageWidget, self).__init__(*args, **kwargs)
            
    is_hidden = False
    button_js = u'''
    <script type="text/javascript">
    
    var changed = false
    
    django.jQuery(document).ready(function () {
        django.jQuery("#id_slug").change(function() { changed = true; });
        django.jQuery('#id_title').change(function() { changed = true; });
    })  
    
    trigger_lang_button = function(e, url) {
        // also make sure that we will display the confirm dialog
        // in case users switch tabs while editing plugins
        
        if(django.jQuery("iframe").length){
            changed = true;
        }

        if (changed) {
            var question = gettext("Are you sure you want to change tabs without saving the page first?")
            var answer = confirm(question);
        }else{
            var answer = true;
        }

        if (!answer) {
            return false;
        } else {
            window.location = url;
        }
    }
  
    </script>
    '''
    
    def render(self, name, value, attrs=None):
        
        hidden_input = super(LanguageWidget, self).render(name, value, attrs=attrs)
        lang_dict = dict(settings.LANGUAGES)
        
        current_languages = []
        translation_of_obj = self.translation_of_obj
        if translation_of_obj and translation_of_obj.pk:
            info = translation_pool.get_info(translation_of_obj.__class__)
            translation_of_obj = translation_pool.annotate_with_translations(translation_of_obj)
            for translation in translation_of_obj.translations:
                current_languages.append(getattr(translation, info.language_field))
                
        buttons = []
        for lang in settings.LANGUAGES:
            current_lang = lang[0] == value
            language_exists = lang[0] in current_languages
            button_classes = u'class="button%s"' % (
                current_lang and ' simple-translation-current' or language_exists and ' simple-translation-exists' or '',
            )
            disabled = current_lang and ' disabled="disabled"' or ''
            buttons.append(u''' <input onclick="trigger_lang_button(this,'./?language=%s');"
                %s name="%s" value="%s" type="button"%s />''' % (
                    lang[0], button_classes, lang[0], lang[1], disabled
                )
            )
                     
        if self.translation_obj.pk and len(current_languages) > 1:
            lang_descr = _('Delete %s translation') % force_unicode(lang_dict[str(value)])
            buttons.append(u'''<p class="deletelink-box simple-translation-delete"><a href="delete-translation/?language=%s" class="deletelink deletetranslation">%s</a></p>''' % (value, lang_descr))
                    
        tabs = u"""%s%s%s""" % (self.button_js, hidden_input, u''.join(buttons))

        return mark_safe(tabs)



from django.conf import settings
from cms.utils.placeholder import PlaceholderNoAction
from cms.models import CMSPlugin

LANG_DICT = dict(settings.LANGUAGES)

class SimpleTranslationPlaceholderActions(PlaceholderNoAction):
    can_copy = True

    def copy(self, target_placeholder, source_language, fieldname, model, target_language, **kwargs):

        plugins = list(target_placeholder.get_plugins().filter(language=source_language))
        ptree = []
        new_plugins = []
        for p in plugins:
            new_plugins.append(p.copy_plugin(target_placeholder, target_language, ptree))
        return new_plugins
    
    def get_copy_languages(self, placeholder, model, fieldname, **kwargs):
        language_codes = CMSPlugin.objects.filter(placeholder=placeholder).distinct().values_list('language', flat=True)
        return [(lc, LANG_DICT.get(lc)) for lc in language_codes] 



from simple_translation.test.testapp.models import EntryTitle
from django.conf.urls.defaults import *

entry_info_dict = {
    'queryset': EntryTitle.objects.all(),
    'date_field': 'pub_date',
    'allow_future': True
}

entry_info_tagged_dict = {
    'queryset_or_model': EntryTitle.objects.all(),
}

entry_info_month_dict = {
    'queryset': EntryTitle.objects.all(),
    'date_field': 'pub_date',
    'month_format': '%m',
}

entry_info_detail_dict = dict(entry_info_month_dict, slug_field='entrytitle__slug')

urlpatterns = patterns('',
    (r'^$', 'django.views.generic.date_based.archive_index', entry_info_dict, 'entry_archive_index'),
    
    (r'^(?P<year>\d{4})/$', 
        'django.views.generic.date_based.archive_year', entry_info_dict, 'entry_archive_year'),
    
    (r'^(?P<year>\d{4})/(?P<month>\d{2})/$', 
        'django.views.generic.date_based.archive_month', entry_info_month_dict, 'entry_archive_month'),
    
    (r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/$', 
        'django.views.generic.date_based.archive_day', entry_info_month_dict, 'entry_archive_day'),
    
    (r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/(?P<slug>[-\w]+)/$', 
        'django.views.generic.date_based.object_detail', entry_info_detail_dict, 'entry_detail')
    
)


from django.db import models
from django.conf import settings

class TranslationAllreadyRegistered(Exception):
    pass

class TranslationOptions(object):
    
    def __init__(self, options={}):
        self.language_field = options.get('language_field', 'language')
        self.translation_of_model = options.get('translation_of_model')
        self.translated_model = options.get('translated_model')
        self.translation_of_field = options.get('translation_of_field')
        self.translations_of_accessor = options.get('translations_of_accessor')
        self.translation_join_filter = options.get('translation_join_filter')
        
class TranslationPool(object):
    
    discovered = False
    translated_models_dict = {}
    translation_models_dict = {}
    
    def discover_translations(self):        
        if self.discovered:
            return
        for app in settings.INSTALLED_APPS:
            __import__(app, {}, {}, ['simple_translate'])
        self.discovered = True    

    def get_info(self, model):
        self.discover_translations()
        if model in self.translated_models_dict:
            return self.translated_models_dict[model]
        elif model in self.translation_models_dict:
            return self.translated_models_dict[ \
                self.translation_models_dict[model]
            ]
        
    def register_translation(self, translation_of_model, translated_model, \
        language_field='language'):
        
        assert issubclass(translation_of_model, models.Model) \
            and issubclass(translated_model, models.Model)
        
        if translation_of_model in self.translated_models_dict:
            raise TranslationAllreadyRegistered, \
                "[%s] a translation for this model is already registered" \
                    % translation_of_model.__name__
            
        options = {}    
        options['translated_model'] = translated_model
        
        opts = translation_of_model._meta
        for rel in opts.get_all_related_objects():
            if rel.model == translated_model:
                options['translation_of_field'] = rel.field.name
                options['translations_of_accessor'] = rel.get_accessor_name()

        options['translation_join_filter'] = translated_model.__name__.lower()          
        options['language_field'] = language_field     
        
        self.translated_models_dict[translation_of_model] = TranslationOptions(options)
        # keep track both ways
        self.translation_models_dict[translated_model] = translation_of_model

    def unregister_translation(self, translation_of_model):
        info = self.get_info(translation_of_model)
        del self.translation_models_dict[info.translated_model]
        del self.translated_models_dict[translation_of_model]
    
    def annotate_with_translations(self, list_or_instance):
        
        self.discover_translations()
        if not list_or_instance:
            return list_or_instance
        if isinstance(list_or_instance, models.Model):
            model = list_or_instance.__class__
            instance = list_or_instance
            
            if self.is_registered_translation(model):
                instance = getattr(list_or_instance, \
                    self.get_info(model).translation_of_field)
                    
            list_or_instance.translations = list(getattr(instance, \
            	self.get_info(model).translations_of_accessor).all())
            return list_or_instance
        else:
            result_list = list_or_instance
            if not len(result_list):
                return result_list
                
            model = list_or_instance[0].__class__
            
            info = self.get_info(model)
            
               
            id_list = [r.pk for r in result_list]
            pk_index_map = dict([(pk, index) for index, pk in enumerate(id_list)])
            
            translations = info.translated_model.objects.filter(**{
                info.translation_of_field + '__in': id_list
            })
            
            for obj in translations:
                index = pk_index_map[getattr(obj, info.translation_of_field + '_id')]
                if not hasattr(result_list[index], 'translations'):
                    result_list[index].translations = []
                result_list[index].translations.append(obj)
        
        return result_list
    
    def is_registered_translation(self, model):
        self.discover_translations()
        if model in self.translation_models_dict:
            return True
        return False
        
    def is_registered(self, model):
        self.discover_translations()
        if model in self.translated_models_dict:
            return True
        return False
            
translation_pool = TranslationPool()



from setuptools import setup, find_packages
import os

setup(
    name='simple-translation',
    version='0.7.9',
    description='This is a translation app/plugin with support for django-cms 2.1 plugins in the admin',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README')).read(),
    author='Oyvind Saltvik',
    author_email='oyvind.saltvik@gmail.com',
    url='http://github.com/fivethreeo/simple-translation/',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    test_suite = 'simple_translation.test.run_tests.run_tests',
    include_package_data=True,
    zip_safe=False
)




import sys

def run_tests():
    
    from django.conf import settings
    
    settings.configure(
        DATABASES = {
            'default':  {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': 'simple_translation.db'
            }
        },
        
        TEMPLATE_CONTEXT_PROCESSORS = (
            "django.core.context_processors.auth",
            "django.core.context_processors.i18n",
            "django.core.context_processors.debug",
            "django.core.context_processors.request",
            "django.core.context_processors.media",
            'django.core.context_processors.csrf',
        ),
        
        
        MIDDLEWARE_CLASSES = [
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.doc.XViewMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
        ],
        
        ROOT_URLCONF = 'simple_translation.test.testapp.no_translated_urls',
        
        INSTALLED_APPS = (
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.admin',
            'django.contrib.sites',
            'django.contrib.staticfiles',
            'simple_translation',
            'simple_translation.test.testapp'
        ),
        
        LANGUAGE_CODE = "en",
        
        LANGUAGES = (
            ("en", "English"),
            ("de", "German"),
        ),
        
        SITE_ID = 1
        

    )
    
    from django.test.utils import get_runner

    failures = get_runner(settings)().run_tests(['simple_translation'])
    sys.exit(failures)

if __name__ == '__main__':
    run_tests()